@@ -36,21 +36,17 @@ namespace Banshee.Gui.TrackEditor
 {
     public class BasicTrackDetailsPage : FieldPage, ITrackEditorPage
     {
-        private CheckButton enable_compilation = new CheckButton ();
-        
         public int Order {
             get { return 10; }
         }
-                                    
+
         public string Title {
             get { return Catalog.GetString ("Basic Details"); }
         }
-        
+
         public override void LoadTrack (EditorTrackInfo track)
         {
             base.LoadTrack (track);
-            enable_compilation.Active = track.IsCompilation;
-            OnEnableCompilationToggled (enable_compilation, EventArgs.Empty);
         }
 
         protected override void AddFields ()
@@ -58,58 +54,80 @@ namespace Banshee.Gui.TrackEditor
             HBox box = new HBox ();
             VBox left = EditorUtilities.CreateVBox ();
             VBox right = EditorUtilities.CreateVBox ();
-            
+
             box.PackStart (left, true, true, 0);
             box.PackStart (new VSeparator (), false, false, 12);
             box.PackStart (right, false, false, 0);
             box.ShowAll ();
-            
+
             PackStart (box, false, false, 0);
-            
+
             // Left
-            
-            AddField (left, new TitleEntry (Dialog), null,
+
+            PageNavigationEntry title_entry = new PageNavigationEntry (Dialog);
+            AddField (left, title_entry, null,
                 delegate { return Catalog.GetString ("Track Title:"); },
-                delegate (EditorTrackInfo track, Widget widget) { ((TitleEntry)widget).Text = track.TrackTitle; },
-                delegate (EditorTrackInfo track, Widget widget) { track.TrackTitle = ((TitleEntry)widget).Text; },
+                delegate (EditorTrackInfo track, Widget widget) { ((PageNavigationEntry)widget).Text = track.TrackTitle; },
+                delegate (EditorTrackInfo track, Widget widget) { track.TrackTitle = ((PageNavigationEntry)widget).Text; },
                 FieldOptions.NoSync
             );
-            
-            AddField (left, new TextEntry (), 
+
+            PageNavigationEntry track_artist_entry = new PageNavigationEntry (Dialog, "CoreArtists", "Name");
+            FieldPage.FieldSlot track_artist_slot = AddField (left, track_artist_entry, 
                 Catalog.GetString ("Set all track artists to this value"),
                 delegate { return Catalog.GetString ("Track Artist:"); },
-                delegate (EditorTrackInfo track, Widget widget) { ((TextEntry)widget).Text = track.ArtistName; },
-                delegate (EditorTrackInfo track, Widget widget) { track.ArtistName = ((TextEntry)widget).Text; }
+                delegate (EditorTrackInfo track, Widget widget) { ((PageNavigationEntry)widget).Text = track.ArtistName; },
+                delegate (EditorTrackInfo track, Widget widget) { track.ArtistName = ((PageNavigationEntry)widget).Text; }
             );
-            
-            enable_compilation.Toggled += OnEnableCompilationToggled;
-            AddField (left, enable_compilation, new TextEntry (), null,
-                delegate (EditorTrackInfo track, Widget widget) { 
-                    ((CheckButton)widget).Label = Catalog.GetString ("Album Artist (part of a compilation):"); 
-                    return null; 
+
+            AlbumArtistEntry album_artist_entry = new AlbumArtistEntry (track_artist_slot.SyncButton, 
+                title_entry, track_artist_entry);
+            AddField (left, null, album_artist_entry,
+                Catalog.GetString ("Set all compilation album artists to these values"), null,
+                delegate (EditorTrackInfo track, Widget widget) {
+                    AlbumArtistEntry entry = widget as AlbumArtistEntry;
+                    entry.IsCompilation = track.IsCompilation;
+                    entry.Text = track.AlbumArtist;
                 },
-                delegate (EditorTrackInfo track, Widget widget) { ((TextEntry)widget).Text = track.AlbumArtist; },
-                delegate (EditorTrackInfo track, Widget widget) { track.AlbumArtist = ((TextEntry)widget).Text; }
+                delegate (EditorTrackInfo track, Widget widget) {
+                    AlbumArtistEntry entry = widget as AlbumArtistEntry;
+                    track.IsCompilation = entry.IsCompilation;
+                    track.AlbumArtist = entry.Text;
+                }
             );
-            
-            AddField (left, new TextEntry (), 
+
+            AddField (left, new TextEntry ("CoreAlbums", "Title"), 
                 Catalog.GetString ("Set all album titles to this value"),
                 delegate { return Catalog.GetString ("Album Title:"); },
                 delegate (EditorTrackInfo track, Widget widget) { ((TextEntry)widget).Text = track.AlbumTitle; },
                 delegate (EditorTrackInfo track, Widget widget) { track.AlbumTitle = ((TextEntry)widget).Text; }
             );
-            
+
             AddField (left, new GenreEntry (), 
                 Catalog.GetString ("Set all genres to this value"),
                 delegate { return Catalog.GetString ("Genre:"); },
                 delegate (EditorTrackInfo track, Widget widget) { ((GenreEntry)widget).Value = track.Genre; },
                 delegate (EditorTrackInfo track, Widget widget) { track.Genre = ((GenreEntry)widget).Value; }
             );
-            
+
             // Right
-            
-            AddField (right, new RangeEntry (Catalog.GetString ("of")), 
-                Catalog.GetString ("Automatically set track number and count"),
+
+            /* Translators: "of" is the word beteen a track/disc number and the total count. */
+            AddField (right, new RangeEntry (Catalog.GetString ("of"), !MultipleTracks 
+                ? null as RangeEntry.RangeOrderClosure
+                : delegate (RangeEntry entry) {
+                    for (int i = 0, n = Dialog.TrackCount; i < n; i++) {
+                        EditorTrackInfo track = Dialog.LoadTrack (i);
+                        track.TrackNumber = i + 1;
+                        track.TrackCount = n;
+                        
+                        if (Dialog.CurrentTrackIndex == i) {
+                            entry.From.Value = track.TrackNumber;
+                            entry.To.Value = track.TrackCount;
+                        }
+                    }
+                }, Catalog.GetString ("Automatically set track number and count")), 
+                null,
                 delegate { return Catalog.GetString ("Track Number:"); },
                 delegate (EditorTrackInfo track, Widget widget) {
                     RangeEntry entry = (RangeEntry)widget;
@@ -123,9 +141,10 @@ namespace Banshee.Gui.TrackEditor
                 },
                 FieldOptions.NoSync
             );
-            
+
             AddField (right, new RangeEntry (Catalog.GetString ("of")), 
-                Catalog.GetString ("Automatically set disc number and count"),
+                // Catalog.GetString ("Automatically set disc number and count"),
+                Catalog.GetString ("Set all disc numbers and counts to these values"),
                 delegate { return Catalog.GetString ("Disc Number:"); },
                 delegate (EditorTrackInfo track, Widget widget) {
                     RangeEntry entry = (RangeEntry)widget;
@@ -137,41 +156,30 @@ namespace Banshee.Gui.TrackEditor
                     track.DiscNumber = (int)entry.From.Value;
                     track.DiscCount = (int)entry.To.Value;
                 },
-                FieldOptions.NoSync
+                FieldOptions.None
             );
-            
+
             Label year_label = EditorUtilities.CreateLabel (null);
-            enable_compilation.SizeAllocated += delegate { year_label.HeightRequest = enable_compilation.Allocation.Height; };
-            AddField (right, year_label, new SpinButtonEntry (1000, 3000, 1), 
+            album_artist_entry.LabelWidget.SizeAllocated += delegate { 
+                year_label.HeightRequest = album_artist_entry.LabelWidget.Allocation.Height; 
+            };
+            SpinButtonEntry year_entry = new SpinButtonEntry (0, 3000, 1);
+            year_entry.Numeric = true;
+            AddField (right, year_label, year_entry,
                 Catalog.GetString ("Set all years to this value"),
                 delegate { return Catalog.GetString ("Year:"); },
                 delegate (EditorTrackInfo track, Widget widget) { ((SpinButtonEntry)widget).Value = track.Year; },
-                delegate (EditorTrackInfo track, Widget widget) { track.Year = (int)((SpinButtonEntry)widget).Value; },
+                delegate (EditorTrackInfo track, Widget widget) { track.Year = ((SpinButtonEntry)widget).ValueAsInt; },
                 FieldOptions.Shrink
             );
-            
+
             AddField (right, new RatingEntry (), 
                 Catalog.GetString ("Set all ratings to this value"),
                 delegate { return Catalog.GetString ("Rating:"); },
                 delegate (EditorTrackInfo track, Widget widget) { ((RatingEntry)widget).Value = track.Rating; },
                 delegate (EditorTrackInfo track, Widget widget) { track.Rating = ((RatingEntry)widget).Value; },
-                FieldOptions.Shrink
+                FieldOptions.Shrink | FieldOptions.NoSync
             );
         }
-        
-        private void OnEnableCompilationToggled (object o, EventArgs args)
-        {
-            CheckButton check = (CheckButton)o;
-            
-            if (CurrentTrack != null) {
-                CurrentTrack.IsCompilation = check.Active;
-            }
-            
-            foreach (Widget child in ((Container)check.Parent).Children) {
-                if (child != check) {
-                    child.Sensitive = check.Active;
-                }
-            }
-        }
     }
 }
